/*
 * sound mixing (combine two sounds together)
 */
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <fcntl.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>
#include <multimedia/audio_hdr.h>
#include <multimedia/ulaw2linear.h>

#include "xplayer.h"
#include "xp_defs.h"

/*
 * Externs
 */

extern	void		dsp_msg();	/* xp_utils.c		*/
extern	void		frm_busy();	/* xp_utils.c		*/
extern	void		xp_log();	/* xp_utils.c		*/
extern	void		speed();	/* xp_speed.c		*/

extern	int	nb_loaded;	/* nombre de sons en memoire	*/
extern	int	nb_selected;	/* nombre de sons a jouer	*/
extern	Sau_desc	au[MAX_AU];	/* loaded sounds	*/

/*
 * Globals
 */
/*
 * adjust the values of sliders and num fields
 */

void		au_sldmix(sld)
int		sld;
{
  int		val, otherval;
  
  switch(sld)
  {
    case 1 :
      val = (int) xv_get(Xplayer_pop_mix->sld_mix1, PANEL_VALUE);
      xv_set(Xplayer_pop_mix->num_mix1,
      			PANEL_VALUE, val,
      			NULL);
      			
      otherval = 100 -val;
      xv_set(Xplayer_pop_mix->sld_mix2,
      			PANEL_VALUE, otherval,
      			NULL);
     
      xv_set(Xplayer_pop_mix->num_mix2,
      			PANEL_VALUE, otherval,
      			NULL);      			
      break;
    
    case 2 :
      val = (int) xv_get(Xplayer_pop_mix->sld_mix2, PANEL_VALUE);
      xv_set(Xplayer_pop_mix->num_mix2,
      			PANEL_VALUE, val,
      			NULL);
      			
      otherval = 100 -val;
      xv_set(Xplayer_pop_mix->sld_mix1,
      			PANEL_VALUE, otherval,
      			NULL);
     
      xv_set(Xplayer_pop_mix->num_mix1,
      			PANEL_VALUE, otherval,
      			NULL); 
      break;
  }
}

void		au_nummix(sld)
int		sld;
{
  int		val, otherval;
  
  switch(sld)
  {
    case 1 :
      val = (int) xv_get(Xplayer_pop_mix->num_mix1, PANEL_VALUE);
      xv_set(Xplayer_pop_mix->sld_mix1,
      			PANEL_VALUE, val,
      			NULL);
      			
      otherval = 100 -val;
      xv_set(Xplayer_pop_mix->sld_mix2,
      			PANEL_VALUE, otherval,
      			NULL);
     
      xv_set(Xplayer_pop_mix->num_mix2,
      			PANEL_VALUE, otherval,
      			NULL);      			
      break;
    
    case 2 :
      val = (int) xv_get(Xplayer_pop_mix->num_mix2, PANEL_VALUE);
      xv_set(Xplayer_pop_mix->sld_mix2,
      			PANEL_VALUE, val,
      			NULL);
      			
      otherval = 100 -val;
      xv_set(Xplayer_pop_mix->sld_mix1,
      			PANEL_VALUE, otherval,
      			NULL);
     
      xv_set(Xplayer_pop_mix->num_mix1,
      			PANEL_VALUE, otherval,
      			NULL); 
      break;
  }

}

/*
 * create a sound
 */
void		au_mix()
{
  char		temp[120];
  unsigned char	*pdata_a, *pdata_b, *pdata;
  Xv_opaque	set_snd;
  int		slot1, slot2, slot3, mix_a, mix_b;
  int		size, size1, size2, size_b;
  int		i, ind;
  double	fact;
  short		mag, curmag;
      
  speed(0);

  dsp_msg("Mixing sounds");
  xv_set(Xplayer_win_xplayer->btn_log,
		PANEL_INACTIVE, FALSE,
		NULL);
  xv_set(Xplayer_pop_log->pop_log,
		XV_SHOW, TRUE,
		NULL);
  
  /* get the mixing specifications				*/
  
  slot1 = (int) xv_get(Xplayer_pop_mix->set_mslot1, PANEL_VALUE);
  slot2 = (int) xv_get(Xplayer_pop_mix->set_mslot2, PANEL_VALUE);
  slot3 = (int) xv_get(Xplayer_pop_mix->set_mslot3, PANEL_VALUE);
   
  sprintf(temp, "\nMixing sound in slot %d\n", slot3 + 1);
  xp_log(temp);
  
  if((au[slot1].selected == E_NO_SOUND)
  		|| (au[slot2].selected == E_NO_SOUND))
  {
    sprintf(temp,"*\tYou must select two slots containing sounds\n");
    xp_log(temp);
    return;
  }
  if(slot1 == slot2)
  {
    sprintf(temp,"*\tYou must select two different slots\n");
    xp_log(temp);
    return;
  }
  if((slot3 == slot1) || (slot3 == slot2))
  {
    sprintf(temp,
      "*\tThe destination slot must be different\n\tfrom %d and %d\n",
      slot1+1, slot2+1);
    xp_log(temp);
    return;
  }
  
  frm_busy(Xplayer_pop_mix->pop_mix, TRUE);
  
  /* initialize the descriptor				*/
  
  if(au[slot3].selected == E_NO_SOUND)	/* no sound in this slot*/
    nb_loaded++;
  else
  {
    if(au[slot3].selected == E_ON)
      nb_selected -= 1;
      
    au[slot3].selected = E_NO_SOUND;
    (void) free((char *) au[slot3].data);
    /* no need to initialize the rest, it will be overwritten	*/
  }
  (au[slot3].path)[0] = (au[slot3].name)[0] = '\0';
  sprintf(au[slot3].name, "mixed%d%d%d.au",
  		 slot1+1, slot2+1, slot3+1);
  sprintf(au[slot3].info, "Mixed sounds");
  au[slot3].desc.sample_rate = SMP_RATE;
  au[slot3].desc.encoding = AUDIO_ENCODING_ULAW;
  au[slot3].desc.bytes_per_unit = 1;
  au[slot3].desc.samples_per_unit = 1;
  au[slot3].desc.channels = 1;

  size1 = au[slot1].desc.data_size;
  size2 = au[slot2].desc.data_size;
  if(size1 < size2)
  {
    size = size2;
    size_b = size1;
    pdata_a = au[slot2].data;
    pdata_b = au[slot1].data;
    mix_a = (int) xv_get(Xplayer_pop_mix->num_mix2, PANEL_VALUE);
    mix_b = (int) xv_get(Xplayer_pop_mix->num_mix1, PANEL_VALUE);
  }
  else
  {
    size = size1;
    size_b = size2;
    pdata_a = au[slot1].data;
    pdata_b = au[slot2].data;
    mix_a = (int) xv_get(Xplayer_pop_mix->num_mix1, PANEL_VALUE);
    mix_b = (int) xv_get(Xplayer_pop_mix->num_mix2, PANEL_VALUE);
  };
  mix_a *= 2;		/* this will allow to mix a low magnitude	*/
  mix_b *= 2;		/* sound with a high mag one more easily	*/
    
  
  /* allocate a buffer, shrinking if request is too big	*/
   
  au[slot3].desc.data_size = size;
  do 
  {
    au[slot3].data = (unsigned char *)malloc(size);
  } while ((au[slot3].data == NULL) && (size = size - (size / 8)));
    
  if(size != au[slot3].desc.data_size)
  {
    /* Not enough memory to store the whole sound		*/
    sprintf(temp, "%.2f seconds of data truncated from '%s'.",
		 audio_bytes_to_secs(&au[slot3].desc,
		 (size - au[slot3].desc.data_size)), au[slot3].name);
    xp_log(temp);
  }
  au[slot3].desc.data_size = size;
  
  /* Create the sound						*/
  
  pdata = au[slot3].data;
  
  /* determine the "normalization" factor (we want to make	*/
  /* sure that the magnitude of the generated sound will never	*/
  /* be bigger than 1.0)					*/
    		
  if((au[slot1].magnitude + au[slot2].magnitude) > 1)
    fact = 200 * (au[slot1].magnitude + au[slot2].magnitude);
  else
    fact = 200.0;
    
  fact = 1/fact;
  
  for(i=0, ind = 0;
      i < size;
      i++, ind = (ind + 1) % size_b, pdata++, pdata_a++)
  {
    *pdata = audio_s2u((short)
    	(fact * (mix_a * audio_u2s(*pdata_a)
    		  + mix_b * audio_u2s(pdata_b[ind]))));
  }
    
  /* determine the magnitude of the sound			*/
  for(i = 0, mag = 0, pdata = au[slot3].data;
        	i < size;
        	i++, pdata++)
  {
    curmag = ABS(audio_u2s(*pdata));
    if(curmag > mag)
      mag = curmag;
  }
  au[slot3].magnitude = audio_s2d(mag);
  
  xv_set(Time_popup_pop_time->pop_time,
		XV_SHOW, TRUE,
		NULL);
		
  /* Display some stats of the sound file, now that it has	*/
  /* been sucessfully mixed					*/
  au[slot3].selected = E_OFF;
  sprintf(temp, "\tInfo : %s\n", au[slot3].info);
  xp_log(temp);
  au[slot3].length = audio_bytes_to_secs(&au[slot3].desc, 
  				au[slot3].desc.data_size);
  sprintf(temp, "\tSize : %d octets\n", au[slot3].desc.data_size);
  xp_log(temp);
  sprintf(temp, "\tLength : %f s\n", au[slot3].length);
  xp_log(temp);
  sprintf(temp, "\tMagnitude : %f\n", au[slot3].magnitude);
  xp_log(temp);
  
  /* Display the file name on the control panel		*/

  switch(slot3 + 1)
  {
    case 1 :
      set_snd = Xplayer_win_xplayer->set_snd1;
      break;
    
    case 2 :
      set_snd = Xplayer_win_xplayer->set_snd2;
      break;
      
    case 3 :
      set_snd = Xplayer_win_xplayer->set_snd3;
      break;
      
    case 4 :
      set_snd = Xplayer_win_xplayer->set_snd4;
      break;
      
    case 5 :
      set_snd = Xplayer_win_xplayer->set_snd5;
      break;
    
    default :
      fputs("Wrong slot\n", stderr);
      quit();
  }
  xv_set(set_snd, PANEL_INACTIVE, FALSE,
        	PANEL_CHOICE_STRING, 0, au[slot3].name,
        	PANEL_VALUE, 0,
        	NULL);
        	
  if(nb_selected == 0)
    stop_play();
  else
    start_play();
      
  dsp_msg("");
  frm_busy(Xplayer_pop_mix->pop_mix, FALSE);
}
